@@ -18,7 +18,7 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-# Copyright (c) 2021 scart97
+# Copyright (c) 2021-2022 scart97
 
 # Original file: https://github.com/NVIDIA/NeMo/blob/main/nemo/collections/asr/parts/quartznet.py
 
@@ -30,19 +30,17 @@ __all__ = [
     "QuartznetBlock",
     "stem",
     "body",
-    "Quartznet_encoder",
+    "QuartznetEncoder",
 ]
 
-from dataclasses import dataclass
 from enum import Enum
-from typing import List
+from typing import List, Tuple
 
 import torch
 from torch import nn
 from torch.nn.common_types import _size_1_t
 
-from thunder.blocks import get_same_padding
-from thunder.utils import default_list
+from thunder.blocks import Masked, MultiSequential, get_same_padding, lengths_to_mask
 
 
 class InitMode(str, Enum):
@@ -69,6 +67,8 @@ def init_weights(m: nn.Module, mode: InitMode = InitMode.xavier_uniform):
     Raises:
         ValueError: Raised when the initial mode is not one of the possible options.
     """
+    if isinstance(m, MaskedConv1d):
+        init_weights(m.conv, mode)
     if isinstance(m, (nn.Conv1d, nn.Linear)):
         if mode == InitMode.xavier_uniform:
             nn.init.xavier_uniform_(m.weight, gain=1.0)
@@ -90,18 +90,96 @@ def init_weights(m: nn.Module, mode: InitMode = InitMode.xavier_uniform):
             nn.init.zeros_(m.bias)
 
 
-class MaskedBatchNorm1d(nn.Module):
-    def __init__(self, out_channels: int):
-        super().__init__()
-        self.layer = nn.BatchNorm1d(out_channels, eps=1e-3, momentum=0.1)
+class MaskedConv1d(nn.Module):
+    __constants__ = ["use_mask", "padding", "dilation", "kernel_size", "stride"]
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
-        if self.training:
-            mask = x == 0.0
-            result = self.layer(x)
-            return torch.masked_fill(result, mask, 0.0)
-        else:
-            return self.layer(x)
+    def __init__(
+        self,
+        in_channels: int,
+        out_channels: int,
+        kernel_size: _size_1_t,
+        stride: _size_1_t = 1,
+        padding: _size_1_t = 0,
+        dilation: _size_1_t = 1,
+        groups: int = 1,
+        bias: bool = False,
+        use_mask: bool = True,
+    ):
+        """Masked Convolution.
+        This module correspond to a 1d convolution with input masking. Arguments to create are the
+        same as nn.Conv1d, but with the addition of use_mask for special behaviour.
+        Args:
+            in_channels: Same as nn.Conv1d
+            out_channels: Same as nn.Conv1d
+            kernel_size: Same as nn.Conv1d
+            stride: Same as nn.Conv1d
+            padding: Same as nn.Conv1d
+            dilation: Same as nn.Conv1d
+            groups: Same as nn.Conv1d
+            bias: Same as nn.Conv1d
+            use_mask: Controls the masking of input before the convolution during the forward.
+        """
+        super().__init__()
+
+        self.use_mask = use_mask
+
+        self.conv = nn.Conv1d(
+            in_channels,
+            out_channels,
+            kernel_size,
+            stride=stride,
+            padding=padding,
+            dilation=dilation,
+            groups=groups,
+            bias=bias,
+        )
+
+        self.padding = self.conv.padding[0]
+        self.dilation = self.conv.dilation[0]
+        self.kernel_size = self.conv.kernel_size[0]
+        self.stride = self.conv.stride[0]
+
+    def get_seq_len(self, lengths: torch.Tensor) -> torch.Tensor:
+        """Get the lengths of the inputs after the convolution operation is applied.
+        Args:
+            lengths: Original lengths of the inputs
+        Returns:
+            Resulting lengths after the convolution
+        """
+        return (
+            torch.div(
+                lengths + 2 * self.padding - self.dilation * (self.kernel_size - 1) - 1,
+                self.stride,
+                rounding_mode="floor",
+            )
+            + 1
+        )
+
+    def mask_fill(self, x: torch.Tensor, lengths: torch.Tensor) -> torch.Tensor:
+        """Mask the input based on it's respective lengths.
+        Args:
+            x: Signal to be processed, of shape (batch, features, time)
+            lengths: Lenghts of each element in the batch of x, with shape (batch)
+        Returns:
+            The masked signal
+        """
+        mask = lengths_to_mask(lengths, x.shape[-1])
+        return x.masked_fill(~mask.unsqueeze(1), 0)
+
+    def forward(
+        self, x: torch.Tensor, lengths: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
+        """Forward method
+        Args:
+            x: Signal to be processed, of shape (batch, features, time)
+            lengths: Lenghts of each element in the batch of x, with shape (batch)
+        Returns:
+            Both the signal processed by the convolution and the resulting lengths
+        """
+        if self.use_mask:
+            x = self.mask_fill(x, lengths)
+        out = self.conv(x)
+        return out, self.get_seq_len(lengths)
 
 
 def _get_conv_bn_layer(
@@ -114,14 +192,14 @@ def _get_conv_bn_layer(
 
     if separable:
         layers = [
-            nn.Conv1d(
+            MaskedConv1d(
                 in_channels,
                 in_channels,
                 kernel_size,
                 groups=in_channels,
                 **conv_kwargs,
             ),
-            nn.Conv1d(
+            MaskedConv1d(
                 in_channels,
                 out_channels,
                 kernel_size=1,
@@ -133,7 +211,7 @@ def _get_conv_bn_layer(
         ]
     else:
         layers = [
-            nn.Conv1d(
+            MaskedConv1d(
                 in_channels,
                 out_channels,
                 kernel_size,
@@ -141,13 +219,13 @@ def _get_conv_bn_layer(
             )
         ]
 
-    layers.append(MaskedBatchNorm1d(out_channels))
+    layers.append(Masked(nn.BatchNorm1d(out_channels, eps=1e-3, momentum=0.1)))
 
     return layers
 
 
 def _get_act_dropout_layer(drop_prob: float = 0.2):
-    return [nn.ReLU(True), nn.Dropout(p=drop_prob)]
+    return [Masked(nn.ReLU(True)), Masked(nn.Dropout(p=drop_prob))]
 
 
 class QuartznetBlock(nn.Module):
@@ -168,15 +246,15 @@ class QuartznetBlock(nn.Module):
         dense residual used on Jasper is not supported here, and masked convolutions were also removed.
 
         Args:
-            in_channels : Number of input channels
-            out_channels : Number of output channels
-            repeat : Repetitions inside block.
-            kernel_size : Kernel size.
-            stride : Stride of each repetition.
-            dilation : Dilation of each repetition.
-            dropout : Dropout used before each activation.
-            residual : Controls the use of residual connection.
-            separable : Controls the use of separable convolutions.
+            in_channels: Number of input channels
+            out_channels: Number of output channels
+            repeat: Repetitions inside block.
+            kernel_size: Kernel size.
+            stride: Stride of each repetition.
+            dilation: Dilation of each repetition.
+            dropout: Dropout used before each activation.
+            residual: Controls the use of residual connection.
+            separable: Controls the use of separable convolutions.
         """
         super().__init__()
 
@@ -217,12 +295,12 @@ class QuartznetBlock(nn.Module):
             )
         )
 
-        self.mconv = nn.Sequential(*conv)
+        self.mconv = MultiSequential(*conv)
 
         if residual:
             stride_residual = stride if stride[0] == 1 else stride[0] ** repeat
 
-            self.res = nn.Sequential(
+            self.res = MultiSequential(
                 *_get_conv_bn_layer(
                     in_channels,
                     out_channels,
@@ -234,34 +312,37 @@ class QuartznetBlock(nn.Module):
         else:
             self.res = None
 
-        self.mout = nn.Sequential(*_get_act_dropout_layer(drop_prob=dropout))
+        self.mout = MultiSequential(*_get_act_dropout_layer(drop_prob=dropout))
 
-    def forward(self, x: torch.Tensor) -> torch.Tensor:
+    def forward(
+        self, x: torch.Tensor, lengths: torch.Tensor
+    ) -> Tuple[torch.Tensor, torch.Tensor]:
         """
         Args:
-            x : Tensor of shape (batch, features, time) where #features == inplanes
+            x: Tensor of shape (batch, features, time) where #features == inplanes
 
         Returns:
             Result of applying the block on the input, and corresponding output lengths
         """
 
         # compute forward convolutions
-        out = self.mconv(x)
+        out, lengths_out = self.mconv(x, lengths)
 
         # compute the residuals
         if self.res is not None:
-            res_out = self.res(x)
+            res_out, _ = self.res(x, lengths)
             out = out + res_out
 
         # compute the output
-        return self.mout(out)
+        out, lengths_out = self.mout(out, lengths_out)
+        return out, lengths_out
 
 
 def stem(feat_in: int) -> QuartznetBlock:
     """Creates the Quartznet stem. That is the first block of the model, that process the input directly.
 
     Args:
-        feat_in : Number of input features
+        feat_in: Number of input features
 
     Returns:
         Quartznet stem block
@@ -283,9 +364,9 @@ def body(
     """Creates the body of the Quartznet model. That is the middle part.
 
     Args:
-        filters : List of filters inside each block in the body.
-        kernel_size : Corresponding list of kernel sizes for each block. Should have the same length as the first argument.
-        repeat_blocks : Number of repetitions of each block inside the body.
+        filters: List of filters inside each block in the body.
+        kernel_size: Corresponding list of kernel sizes for each block. Should have the same length as the first argument.
+        repeat_blocks: Number of repetitions of each block inside the body.
 
     Returns:
         List of layers that form the body of the network.
@@ -315,33 +396,24 @@ def body(
     return layers
 
 
-@dataclass
-class EncoderConfig:
-    """Configuration to create [`Quartznet_encoder`][thunder.quartznet.blocks.Quartznet_encoder]
-
-    Attributes:
-        feat_in: Number of input features to the model. defaults to 64.
-        filters: List of filter sizes used to create the encoder blocks. defaults to [256, 256, 512, 512, 512].
-        kernel_sizes: List of kernel sizes corresponding to each filter size. defaults to [33, 39, 51, 63, 75].
-        repeat_blocks: Number of repetitions of each block. defaults to 1.
-    """
-
-    feat_in: int = 64
-    filters: List[int] = default_list([256, 256, 512, 512, 512])
-    kernel_sizes: List[int] = default_list([33, 39, 51, 63, 75])
-    repeat_blocks: int = 1
-
-
-def Quartznet_encoder(cfg: EncoderConfig = EncoderConfig()) -> nn.Module:
+def QuartznetEncoder(
+    feat_in: int = 64,
+    filters: List[int] = [256, 256, 512, 512, 512],
+    kernel_sizes: List[int] = [33, 39, 51, 63, 75],
+    repeat_blocks: int = 1,
+) -> nn.Module:
     """Basic Quartznet encoder setup.
     Can be used to build either Quartznet5x5 (repeat_blocks=1) or Quartznet15x5 (repeat_blocks=3)
 
     Args:
-        cfg: required config to create instance
+        feat_in: Number of input features to the model.
+        filters: List of filter sizes used to create the encoder blocks.
+        kernel_sizes: List of kernel sizes corresponding to each filter size.
+        repeat_blocks: Number of repetitions of each block.
     Returns:
         Pytorch model corresponding to the encoder.
     """
-    return nn.Sequential(
-        stem(cfg.feat_in),
-        *body(cfg.filters, cfg.kernel_sizes, cfg.repeat_blocks),
+    return MultiSequential(
+        stem(feat_in),
+        *body(filters, kernel_sizes, repeat_blocks),
     )
